@@ -2,7 +2,7 @@
 BEGIN { $diagnostics::PRETTY = 1 }
 
 package Pat::Logger;
-our $VERSION = '3.2';
+our $VERSION = '3.3';
 
 use Time::localtime;
 
@@ -11,7 +11,7 @@ use diagnostics;
 use Fcntl ':flock';
 use Term::ANSIColor qw (:constants);
 
-use fields qw ( Error_Message Error_Color Debug_Log First_Message Separator time_pack sub_pack line_pack msg_pack );
+use fields qw (msg_split_width Mode Error_Message Error_Color Debug_Log First_Message Separator time_pack sub_pack line_pack msg_pack );
 
 
 # ------------------------
@@ -24,9 +24,10 @@ sub separate();
 
 sub eval_time();
 sub _get_header();
-sub _pack_str($$$$);
+sub _pack_str($$$);
 sub _print_to_file($);
-sub _get_debug_params();
+sub _split_message( $ );
+sub _get_debug_params( );
 
 
 # Global variables:
@@ -56,7 +57,7 @@ sub new($)
 {
     my ( $class, $debug_file ) = @_;
     my $self;
-
+   
    
     # Bless object:    
     $self = fields::new $class;
@@ -73,7 +74,14 @@ sub new($)
     $self->{sub_pack}  = $sub_pack;
     $self->{line_pack} = $line_pack;
     $self->{msg_pack}  = $msg_pack;
-
+    
+    foreach ( $time_pack, $sub_pack, $line_pack )
+    {
+        $_ =~ /^\w(\d+)$/;
+        $self->{msg_split_width} += $1;
+    }
+    
+    #print "Msg length is $self->{msg_split_width}\n";
     return $self;
 }
 
@@ -98,6 +106,7 @@ sub debug_message($)
     my ( $package, $sub, $line, $time ) ;
     my $debug_message = '';
     my $rc;
+    my @split_message;
     
     # If it's an error message, we raise the flag.
     $self->{Error_Message} = 1 if defined $error_flag;
@@ -121,20 +130,48 @@ sub debug_message($)
     print STDERR $debug_message;
 
     # Now the message parameters;
-    $debug_message = $self->_pack_str( $time, $package, $sub, $line, $message );
+    $debug_message = $self->_pack_str( $time, $package, $sub, $line );
     $self->_print_to_file( $debug_message ) if defined $self->{Debug_Log};
     print STDERR $debug_message;
+   
     
     # Now we pack and print the actual message.
     # We do this here because we want to play with the colors:
-    $debug_message = pack ( $self->{msg_pack} , $message ) . "\n";
-    $self->_print_to_file( $debug_message ) if defined $self->{Debug_Log};
+    @split_message = $self->_split_message( $message );
+    $message = shift @split_message;
+    $message .= "\n";
     
     if ( $self->{Error_Message} ) {
-        print STDERR BLINK RED $debug_message, RESET;
+        print STDERR BLINK RED $message, RESET;
     } else {
-        print STDERR $debug_message;
+        print STDERR $message;
     }
+    $self->_print_to_file( $message ) if defined $self->{Debug_Log};
+    
+
+    
+    my $gap = ' ' x $self->{msg_split_width};
+    foreach $message ( @split_message )
+    {
+        $message .= "\n";
+        if ( $self->{Error_Message} ) {
+            print STDERR BLINK RED  $gap . $message   , RESET;
+        } else {
+            print STDERR $gap . $message   ;
+        }
+        $self->_print_to_file( $gap . $message   ) if defined $self->{Debug_Log};
+    }
+    
+    
+    
+    #$debug_message = pack ( $self->{msg_pack} , $message ) . "\n";
+    #$self->_print_to_file( $debug_message ) if defined $self->{Debug_Log};
+    
+    #if ( $self->{Error_Message} ) {
+    #    print STDERR BLINK RED $debug_message, RESET;
+    #} else {
+    #    print STDERR $debug_message;
+    #}
    
 
     # Reset error flag:
@@ -146,17 +183,63 @@ sub debug_message($)
 
 
 
-
-sub _print_to_file($)
+sub _split_message( $ )
 {
     my $self = shift;
     my $message = shift;
+    my @split_message;
+    
+    my $length = $1 if ( $self->{msg_pack} =~ /^.(\d+)$/ );
+    if ( length $message > $length )
+    {
+        my $tmp_1;
+        my $tmp_2;
+        
+
+        while ( $message =~ /^(.{$length})(.*)$/ )
+        {
+            $tmp_1 = $1;
+            $tmp_2 = $2;
+
+            unless ( $tmp_2 =~ /^\s/ )
+            {
+                if ( $tmp_1 =~ /^(.*)\s+(.*)$/ )
+                {
+                    $tmp_1 = $1;
+                    $tmp_2 = $2 . $tmp_2;
+                }
+            }
+            $tmp_2 =~ s/^\s+//;
+            
+            push @split_message, $tmp_1;
+            $message = $tmp_2;
+        }
+        push @split_message, $message;
+    }
+    else
+    {
+        push @split_message, $message;
+    }
+    
+    return @split_message;
+}
+
+
+
+
+sub _print_to_file($)
+{
+    no strict 'refs';
+    
+    my $self = shift;
+    my $message = shift;
+    my $FH = $$;
     
     # we are using a log file, so let's create a file handle to it:
     if (  defined $self->{Debug_Log} )
     {
 	# Create File handle to log file:   
-	if (! open (LOG, ">> $self->{Debug_Log}") )
+	if (! open ($FH, ">> $self->{Debug_Log}") )
 	{
 	    print "Can't open $self->{Debug_Log}: $!\n";
 	    return 0;
@@ -164,9 +247,9 @@ sub _print_to_file($)
 	else
 	{
 	    # We lock the file for security reasons:
-	    flock LOG, LOCK_EX;
-	    print LOG $message;
-	    close LOG;
+	    flock $FH, LOCK_EX;
+	    print $FH $message;
+	    close $FH;
 	}
 	    
     }
@@ -185,9 +268,9 @@ sub _get_header()
     my $headers = $self->{Separator};
     
     $headers .= pack $self->{time_pack}, ' TIME';
-    $headers .= pack $self->{sub_pack}, 'METHOD';
+    $headers .= pack $self->{sub_pack},  'METHOD';
     $headers .= pack $self->{line_pack}, 'LINE';
-    $headers .= pack $self->{msg_pack}, 'MESSAGE';
+    $headers .= pack $self->{msg_pack},  'MESSAGE';
     #$headers .= "\n";
     
     $headers .= $self->{Separator};
@@ -210,10 +293,10 @@ sub separate()
 
 
 
-sub _pack_str( $$$$ )
+sub _pack_str( $$$ )
 {
     my $self = shift;
-    my ( $time, $package, $sub, $line, $message ) = @_;
+    my ( $time, $package, $sub, $line ) = @_;
     my $pid;
     my $result = '';
     
@@ -226,13 +309,7 @@ sub _pack_str( $$$$ )
     #$result .= pack ( $self->{msg_pack} , $message );
     #$result .= "\n";
     
-    return $result;
-    
-
-#<<<<<<<<<<<<<<< - $package
-#<<<<<<<<<<<<<<<<<<<<<<<<<<<< - $sub
-#<<<<<<<<< - $line
-#<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< -   $message  
+    return $result;   
 }
 
 
@@ -284,19 +361,19 @@ sub _get_debug_params()
 #
 # Return value  : String holding the outpfile.
 #
-# Dependencies  : HTTP::Date.
+# Dependencies  : localtime.
 # ********************************************************************************************************
 
 sub eval_time()
 {
-	my $tm = localtime;
-	my ( $DAY, $MONTH, $YEAR, $HOUR, $MINUTE, $SECOND ) = ( $tm->mday, $tm->mon, $tm->year, $tm->hour, $tm->min, $tm->sec );
-	$MONTH++;
-	$YEAR += 1900;
-	$YEAR = substr( $YEAR, 2, 2);
-	my $time_and_date = $DAY . '/' . $MONTH . '/' . $YEAR . '-' . $HOUR . ':' . $MINUTE . ':' . $SECOND;
+    my $tm = localtime;
+    my ( $DAY, $MONTH, $YEAR, $HOUR, $MINUTE, $SECOND ) = ( $tm->mday, $tm->mon, $tm->year, $tm->hour, $tm->min, $tm->sec );
+    $MONTH++;
+    $YEAR += 1900;
+    $YEAR = substr( $YEAR, 2, 2);
+    my $time_and_date = $DAY . '/' . $MONTH . '/' . $YEAR . '-' . $HOUR . ':' . $MINUTE . ':' . $SECOND;
 
-	return $time_and_date;
+    return $time_and_date;
 }
 
 
@@ -329,22 +406,21 @@ __END__
     eval { $logger = new Pat::Logger ( $debug_file ) };
     die $@ if $@;
 
-    $logger->debug_message ( "Logger will tell you the package, subroutine and line number" );
-    $logger->debug_message ( 'your debug message originated from' )
-    $logger->debug_message ( "Exception caught: $@" );
+    $logger->debug_message ( 'Logger will tell you the package, subroutine, line number and the time your debug message originated from' );
     $logger->separate;
-    $logger->debug_message ( "This line is separated from the previous one" );
-    $logger->debug_message ( "Fatal Error: $!", 'ERROR' ); # This message will blink in Red.
-
-      
+    $logger->debug_message ( 'This line is separated from the previous one' );
+    $logger->debug_message ( "An error occured", 'ERROR' ); # This message will blink in Red.
+    $logger->debug_message ( "This line is much longer to fit in a single row. Logger will split it nicely, without chopping off words and display it in multiple rows" );
+     
 
 =head1 DESCRIPTION
 
-    The Logger module is a nifty tool to organaize your debug messages.
+    The Logger module is a nifty tool to organaize your debug messages, and thus your program flow.
 
     While writing your code you need a tool to output your debug messages.
-    You want to see where the message originated from (which module, which subroutine and line number),
+    You want to see where the message originated from ( which module, which subroutine and line number and at what time ),
     so you can proceed directly to solving the matter, rather than search for it's location.
+    You want to destinguish between an error message, and yet another flow control message.
     Not only you want to see the messages on screen, you want to have them in a local file as well.
     Logger does just that.
 
@@ -368,26 +444,26 @@ __END__
 
     This method takes two argument - the debug message you wish to log, and the type of the message.
     Currently supported type is 'ERROR'. When the second argument is 'ERROR', the debug message willl appear
-    in blinking Red color ( in case the Terminal e,ulator supports it ).
+    in blinking Red color ( in case your Terminal supports it ).
     
     Upon success - the method returns 1.
     Upon failre  - the method returns 0.
     
     The Logger object does all the work behind the scenes:
-    (1) Grab the package, subroutine name and line number which the message originated from.
+    (1) Grab the time, package, subroutine name and line number which the message originated from.
     (2) Create a nice format with the parameters aforementioned.
     (3) Output it according to object type.
 
 =item * B<separate()>
 
     You may wish to create visual separation between messages.
-    When you invoke separate(), a line consistant of 152 x '-' will be outputed.
+    When you invoke separate(), a line consistant of '-' will be outputed.
 
-    This length is coherent with the length of the format.
+    This length is automatically calculated by Logger.
     
 =head1 WIDTH CONTROL
 
-    The Logger module B<uses pack()> to indent the output.
+    The Logger module uses pack() to indent the output.
     You can control the  width of each field by altering the code:
                          
     my $time_pack = 'A18'; This mean the 'TIME' column is 18 byte long.
