@@ -7,12 +7,17 @@
 
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
+my $loaded;
 
 BEGIN { $| = 1; print "1..1\n"; }
 END {print "not ok 1\n" unless $loaded;}
+
 use lib qw( .. );
 use Pat::Logger;
-use Time::localtime;
+use strict;
+use diagnostics;
+use Fcntl ':flock';
+use Term::ANSIColor qw (:constants);
 
 $loaded = 1;
 print "ok 1\n";
